<?php
/***************************************************************
 *  Copyright notice
 *
 *  (c) 2009-2011 Ingo Renner <ingo@typo3.org>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/


/**
 * A caching backend which stores cache entries by using Memcached.
 *
 * This backend uses the following types of Memcache keys:
 * - tag_xxx
 *   xxx is tag name, value is array of associated identifiers identifier. This
 *   is "forward" tag index. It is mainly used for obtaining content by tag
 *   (get identifier by tag -> get content by identifier)
 * - ident_xxx
 *   xxx is identifier, value is array of associated tags. This is "reverse" tag
 *   index. It provides quick access for all tags associated with this identifier
 *   and used when removing the identifier
 *
 * Each key is prepended with a prefix. By default prefix consists from two parts
 * separated by underscore character and ends in yet another underscore character:
 * - "TYPO3"
 * - Current site path obtained from the PATH_site constant
 * This prefix makes sure that keys from the different installations do not
 * conflict.
 *
 * Note: When using the Memcached backend to store values of more than ~1 MB,
 * the data will be split into chunks to make them fit into the memcached limits.
 *
 * This file is a backport from FLOW3 by Ingo Renner.
 *
 * @package TYPO3
 * @subpackage t3lib_cache
 * @api
 * @version $Id: class.t3lib_cache_backend_memcachedbackend.php 10444 2011-02-11 22:48:05Z lolli $
 */
class tx_libeomemcache_handler_LibeoMemcachedBackendAgressive extends tx_libeomemcache_handler_LibeoMemcachedBackendBase {

    /**
     * @var integer
     */
	protected $cacheVersion;

    /**
     * @var string
     */
	protected $versionKey;

	/**
	 * Initializes the identifier prefix when setting the cache.
	 *
	 * @param t3lib_cache_frontend_Frontend $cache The frontend for this backend
	 * @return void
	 * @author Robert Lemke <robert@typo3.org>
	 * @author Dmitry Dulepov
	 */
	public function setCache(t3lib_cache_frontend_Frontend $cache) {
		parent::setCache($cache);
        $this->versionKey = $this->identifierPrefix."___CACHE_VERSION";
	}

    /**
     * @return string
     */
    private function getIdentifierPrefix(){
        $this->loadCacheVersion();
        return $this->identifierPrefix . "_v" . $this->cacheVersion;
    }

    private function loadCacheVersion(){
        if($this->cacheVersion == 0){
            $this->debug("GET loadCacheVersion()");
            $version = $this->memcache->get($this->versionKey);
            if($version != false){
                $this->cacheVersion = $version;

            }else{
                $this->cacheVersion = 1;
                $this->saveCacheVersion($this->cacheVersion);
            }
        }
    }

    private function saveCacheVersion($version){
        $this->loadCacheVersion();
        $this->debug("SET saveCacheVersion()");
        $this->memcache->set($this->versionKey, $version, null, 0);
    }

	/**
	 * Saves data in the cache.
	 *
	 * @param string An identifier for this specific cache entry
	 * @param string The data to be stored
	 * @param array Tags to associate with this cache entry
	 * @param integer Lifetime of this cache entry in seconds. If NULL is specified, the default lifetime is used. "0" means unlimited liftime.
	 * @return void
	 * @throws t3lib_cache_Exception if no cache frontend has been set.
	 * @throws InvalidArgumentException if the identifier is not valid or the final memcached key is longer than 250 characters
	 * @throws t3lib_cache_exception_InvalidData if $data is not a string
	 * @author Christian Jul Jensen <julle@typo3.org>
	 * @author Karsten Dambekalns <karsten@typo3.org>
	 */
	public function set($entryIdentifier, $data, array $tags = array(), $lifetime = NULL) {
		parent::innerSet($this->getFinalKey($entryIdentifier), $data, $lifetime);
	}

	/**
	 * Loads data from the cache.
	 *
	 * @param string An identifier which describes the cache entry to load
	 * @return mixed The cache entry's content as a string or FALSE if the cache entry could not be loaded
	 * @author Christian Jul Jensen <julle@typo3.org>
	 * @author Karsten Dambekalns <karsten@typo3.org>
	 */
	public function get($entryIdentifier) {
        return parent::innerGet($this->getFinalKey($entryIdentifier), $entryIdentifier);
	}

    protected function getFinalKey($entryIdentifier) {
        return $this->getIdentifierPrefix() . $entryIdentifier;
    }


    /**
	 * Removes all cache entries of this cache.
	 *
	 * @return void
	 * @author Karsten Dambekalns <karsten@typo3.org>
	 */
	public function flush() {
		if (!$this->cache instanceof t3lib_cache_frontend_Frontend) {
			throw new t3lib_cache_Exception('No cache frontend has been set via setCache() yet.', 1204111376);
		}

        $this->loadCacheVersion();
		$this->cacheVersion++;
        $this->saveCacheVersion($this->cacheVersion);
	}

	public function findIdentifiersByTag($tag) {
        return array();
	}

	public function findIdentifiersByTags(array $tags) {
		return array();
	}

    /**
     * @param string $tag The tag the entries must have
     * @return void
     */
    public function flushByTag($tag) {
        error_log("CACHE AGGRESSIVE - Attempted to flushByTag($tag); which is not supported. ");
    }

    /**
     * @param array $tags
     * @return void
     */
    public function flushByTags(array $tags) {
        error_log("CACHE AGGRESSIVE - Attempted to flushByTags(".var_export($tags, true)."); which is not supported. ");
    }



}


if (defined('TYPO3_MODE') && isset($GLOBALS['TYPO3_CONF_VARS'][TYPO3_MODE]['XCLASS']['t3lib/cache/backend/class.t3lib_cache_backend_libeomemcachedbackend.php'])) {
	include_once($GLOBALS['TYPO3_CONF_VARS'][TYPO3_MODE]['XCLASS']['t3lib/cache/backend/class.t3lib_cache_backend_libeomemcachedbackend.php']);
}

?>